#include <iostream>
#include <vector>

#include "TROOT.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "TF1.h"
#include "TF2.h"
#include "TH1.h"
#include "TH2.h"
#include "TMath.h"
#include "TRandom3.h"
#include "TFile.h"
#include "TTree.h"

const Int_t nNodes=2;
const Int_t nVars=2;

Double_t activation(Double_t x){
   return 1./(1+TMath::Exp(-x));
}

Double_t activationDerivative(Double_t x){
   Double_t denominator = (1.0+TMath::Exp(-x))*(1.0+TMath::Exp(-x));
   return TMath::Exp(-x)/denominator;
}

Double_t mlp(Double_t w1[nNodes][nVars+1], Double_t w2[nNodes+1], Double_t x[nVars], TH2D* *fn){
   
   Double_t result=0;
   Double_t arg;
   for (Int_t iNode=0; iNode < nNodes; iNode++){
      arg=0;
      for (Int_t iVar=0; iVar < nVars; iVar++){
         arg+=w1[iNode][iVar]*x[iVar];
      }
      arg+=w1[iNode][nVars];
      fn[iNode]->Fill(x[0],x[1],activation(arg));
      result += w2[iNode] * activation(arg);
   }
   result += w2[nNodes]; 
   result = activation(result);
   return result;
}

void miniMLP4()
{
   gROOT->SetStyle("Plain");
   gStyle->SetOptStat(0);

   TH2D *fn[nNodes+1]; // use the last one as 'output node'
   for (Int_t i=0; i<=nNodes; i++){
      fn[i]=new TH2D(Form("fn%d",i),"Node1",100,-10,10,100,-10,10);
      fn[i]->SetTitle(Form("Node%d",i));
      fn[i]->GetXaxis()->SetTitle("X_{1}");
      fn[i]->GetYaxis()->SetTitle("X_{2}");
      fn[i]->GetXaxis()->SetTitleOffset(1.5);
      fn[i]->GetXaxis()->SetTitleSize(0.06);
      fn[i]->GetYaxis()->SetTitleOffset(1.5);
      fn[i]->GetYaxis()->SetTitleSize(0.05);
      fn[i]->GetXaxis()->SetLabelSize(0.04);
      fn[i]->GetYaxis()->SetLabelSize(0.04);
      fn[i]->GetXaxis()->SetNdivisions(305);
      fn[i]->GetYaxis()->SetNdivisions(305);
   }


   //generate some data
   const Int_t nEvents = 1000;
   Double_t xSignal[nEvents][nVars];
   Double_t xBackgr[nEvents][nVars];
   TRandom3 rand(12345);


   Double_t meanS[nVars];
   Double_t meanB[nVars];
   Double_t sigmaS[nVars];
   Double_t sigmaB[nVars];

   for (Int_t iev=0; iev<nEvents; iev++){
     Double_t select = rand.Uniform();
     if (select < 0.3){
       meanB[0]=-4;   meanB[1]=4;
       sigmaB[0]=3;   sigmaB[1]=3;
     }else if (select < 0.6){
       meanB[0]=0;     meanB[1]=-3;
       sigmaB[0]=6;   sigmaB[1]=3;
     }else{
       meanB[0]=6;     meanB[1]=0;
       sigmaB[0]=2;   sigmaB[1]=2;
     }
     select = rand.Uniform();
     if (select < 0.3){
       meanS[0]=1;    meanS[1]=1;
       sigmaS[0]=1;    sigmaS[1]=1;
     }else if (select < 0.6){
       meanS[0]=2;     meanS[1]=3;
       sigmaS[0]=2;    sigmaS[1]=2;
     }else{
       meanS[0]=3;     meanS[1]=6;
       sigmaS[0]=2;    sigmaS[1]=4;
     }
     for (Int_t iVar=0; iVar<nVars; iVar++){
       xSignal[iev][iVar]=rand.Gaus(meanS[iVar],sigmaS[iVar]);
       xBackgr[iev][iVar]=rand.Gaus(meanB[iVar],sigmaB[iVar]);
     }
   }

   TH2D *hs = new TH2D("hs","",100,-10,10,100,-10,10);
   TH2D *hb = new TH2D("hb","",100,-10,10,100,-10,10);
   hs->SetMarkerColor(4);
   hb->SetMarkerColor(2);
   hs->SetMarkerStyle(20);
   hb->SetMarkerStyle(20);
   hs->SetMarkerSize(1.);
   hb->SetMarkerSize(1.);
   TFile *file = new TFile("MCEvents.root","RECREATE");
   TTree *sig = new TTree("sig","MySig"); 
   TTree *bkg = new TTree("bkg","MyBkg"); 
   Float_t x1s, x2s, x1b,x2b;



   sig->Branch("var0",&x1s,"var0/F");
   sig->Branch("var1",&x2s,"var1/F");
   bkg->Branch("var0",&x1b,"var0/F");
   bkg->Branch("var1",&x2b,"var1/F");

   for (Int_t i=0; i<nEvents; i++){
     hs->Fill(xSignal[i][0],xSignal[i][1]);
     hb->Fill(xBackgr[i][0],xBackgr[i][1]);

     x1s=xSignal[i][0];
     x2s=xSignal[i][1];
     x1b=xBackgr[i][0];
     x2b=xBackgr[i][1];
     sig->Fill();
     bkg->Fill();
   }
   file->Write();
   file->Close();

   Double_t x[nVars];  // bias is constant ==1  
   Double_t w1[nNodes][nVars+1]; // on 'bias'
   Double_t w2[nNodes+1]; // on 'bias'

   w1[0][0] = ..... ; 
   w1[1][0] = ..... ; 
   w1[0][1] = ..... ; 
   w1[1][1] = ..... ; 
   w1[0][2] = ..... ; 
   w1[1][2] = ..... ; 
   
   w2[0] = ..... ; 
   w2[1] = ..... ; 
   w2[2] = ..... ; 


   Double_t x2Step=0.2,x1Step=0.2;
   x[0]=-10.+x1Step/2.;
   x[1]=-10.+x2Step/2.;

   // loop over x,y data to fill a histogram allowing the visualisation of the network 'function'
   for (Int_t iX1=0; iX1<100; iX1++){ 
      x[1]=-10. +  x2Step/2.;
      for (Int_t iY=0; iY<100; iY++){
         // the actual calculation of the 'network' output
         Double_t mlpResult = mlp(w1,w2,x,fn);

         fn[nNodes]->Fill(x[0],x[1],mlpResult);
         x[1]+=x2Step;
      }         
      x[0]+=x1Step;
   }

   TCanvas *c=new TCanvas("c","",900,900);
   c->Divide(2,2);
   c->cd(1);
   fn[0]->Draw("surf");

   c->cd(2);
   fn[1]->Draw("surf");

   c->cd(3);
   fn[nNodes]->DrawClone("surf");

   c->cd(4);
   Double_t contours[1]={0.467};
   hs->Draw("same");
   hb->Draw("same");
   fn[nNodes]->SetLineWidth(7);
   fn[nNodes]->SetLineColor(1);
   fn[nNodes]->SetLineStyle(1);
   fn[nNodes]->SetMarkerColor(1);
   fn[nNodes]->SetContour(1,contours);
   fn[nNodes]->Draw("CONT3 same");

}
