#include <iostream>
#include <vector>

#include "TROOT.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "TF1.h"
#include "TF2.h"
#include "TH1.h"
#include "TH2.h"
#include "TMath.h"


Double_t activation(Double_t x){
   return 1./(1+TMath::Exp(-x));
}

Double_t activationDerivative(Double_t x){
   Double_t denominator = (1.0+TMath::Exp(-x))*(1.0+TMath::Exp(-x));
   return TMath::Exp(-x)/denominator;
}

void miniMLP()
{
   gROOT->SetStyle("Plain");
   gStyle->SetOptStat(0);
 

   TH2D *fn =new TH2D("fn","Node1",100,-10,10,100,-10,10);
   fn->GetXaxis()->SetTitle("X_{1}");
   fn->GetYaxis()->SetTitle("X_{2}");
   fn->GetXaxis()->SetTitleOffset(1.5);
   fn->GetXaxis()->SetTitleSize(0.06);
   fn->GetYaxis()->SetTitleOffset(1.5);
   fn->GetYaxis()->SetTitleSize(0.05);
   fn->GetXaxis()->SetLabelSize(0.04);
   fn->GetYaxis()->SetLabelSize(0.04);
   fn->GetXaxis()->SetNdivisions(305);
   fn->GetYaxis()->SetNdivisions(305);




   Double_t result=0.;
   Double_t arg;
   Double_t x2Step=0.2,x1Step=0.2;
   Double_t x1=-10.+x1Step/2.;
   Double_t x2=-10.+x2Step/2.;

   // loop over x,y data to fill a histogram allowing the visualisation of the network 'function'
   for (Int_t iX1=0; iX1<100; iX1++){ 
      x2=-10. +  x2Step/2.;
      for (Int_t iY=0; iY<100; iY++){
         
         // the actual calculation of the 'network' output
         arg=x1+x2;
         result=activation(arg);
         fn->Fill(x1,x2,result);
         x2+=x2Step;
      }         
      x1+=x1Step;
   }

   TCanvas *c=new TCanvas("c","",900,900);
   c->Divide(2,2);
   c->cd(1);
   fn->Draw("surf");

   c->cd(4);
   Double_t contours[1]={.5};
   fn->SetContour(1,contours);
   fn->Draw("CONT1");


}
